package dynamic_programming;
/**
 * 题目： 188. 买卖股票的最佳时机 IV (maxProfit)
 * 描述：给你一个整数数组 prices 和一个整数 k ，其中 prices[i] 是某支给定的股票在第 i 天的价格。
 * 设计一个算法来计算你所能获取的最大利润。你最多可以完成 k 笔交易。也就是说，你最多可以买 k 次，卖 k 次。
 * 注意：你不能同时参与多笔交易（你必须在再次购买前出售掉之前的股票）。

 * 示例 1：
 输入：k = 2, prices = [2,4,1]
 输出：2
 解释：在第 1 天 (股票价格 = 2) 的时候买入，在第 2 天 (股票价格 = 4) 的时候卖出，这笔交易所能获得利润 = 4-2 = 2 。

 * 链接：https://leetcode.cn/problems/best-time-to-buy-and-sell-stock-iv/
 */
//不会
public class MaxProfit4 {

    public int maxProfit(int k, int[] prices) {
        // 如果没有价格数据，无法交易，利润为 0
        if (prices.length == 0) return 0;

        int n = prices.length;
        // dp[i][j]：第 i 天结束时，处于「状态 j」时能获得的最大利润
        // 状态 j 从 0 到 2*k：
        //   j = 0      ：从未操作过（可以看作初始状态，利润为 0）
        //   j = 2*t-1  ：第 t 次买入后（持有股票）的状态
        //   j = 2*t    ：第 t 次卖出后（空仓）的状态
        int[][] dp = new int[n][2 * k + 1];

        // 初始化：第 0 天买入的那些状态
        // 对于每一笔交易 t（1 ≤ t ≤ k），在第 0 天「买入」后，手里就是 −prices[0]
        for (int t = 1; t <= k; t++) {
            dp[0][2*t - 1] = -prices[0];
            // dp[0][2*t] 默认是 0：还没卖出，自然利润为 0
        }

        // 从第 1 天开始，逐日更新
        for (int i = 1; i < n; i++) {
            // 枚举交易次数
            for (int t = 1; t <= k; t++) {
                int buyState  = 2*t - 1;  // 第 t 次买入后
                int sellState = 2*t;      // 第 t 次卖出后

                // 【转移方程 1：买入状态】
                // 要么：继承昨天已经买入的状态 dp[i-1][buyState]
                // 要么：今天刚用上一次卖出的资金买入——从 dp[i-1][buyState-1] 扣除 prices[i]
                dp[i][buyState] = Math.max(
                        dp[i-1][buyState],
                        dp[i-1][buyState - 1] - prices[i]
                );

                // 【转移方程 2：卖出状态】
                // 要么：继承昨天已经卖出的状态 dp[i-1][sellState]
                // 要么：今天把第 t 次买入的股票卖出——在 dp[i-1][buyState] 上加上 prices[i]
                dp[i][sellState] = Math.max(
                        dp[i-1][sellState],
                        dp[i-1][buyState] + prices[i]
                );
            }
        }

        // 答案：第 n-1 天，完成第 k 次卖出后的最大利润
        return dp[n-1][2*k];
    }
}
