package graph;

import java.util.*;

/**
 * 题目： 433. 最小基因变化  (minMutation)
 * 描述：基因序列可以表示为一条由 8 个字符组成的字符串，其中每个字符都是 'A'、'C'、'G' 和 'T' 之一。
 * 假设我们需要调查从基因序列 start 变为 end 所发生的基因变化。一次基因变化就意味着这个基因序列中的一个字符发生了变化。
 * 例如，"AACCGGTT" --> "AACCGGTA" 就是一次基因变化。
 * 另有一个基因库 bank 记录了所有有效的基因变化，只有基因库中的基因才是有效的基因序列。（变化后的基因必须位于基因库 bank 中）
 * 给你两个基因序列 start 和 end ，以及一个基因库 bank ，请你找出并返回能够使 start 变化为 end 所需的最少变化次数。如果无法完成此基因变化，返回 -1 。
 * 注意：起始基因序列 start 默认是有效的，但是它并不一定会出现在基因库中。
 *
 示例 1：
 输入：start = "AACCGGTT", end = "AAACGGTA", bank = ["AACCGGTA","AACCGCTA","AAACGGTA"]
 输出：2

 * 链接：https://leetcode.cn/problems/minimum-genetic-mutation/

 */
//不会
public class MinMutation {
    /**
     * 准备工作
     *
     * 将基因库 bank 中的有效基因串存入一个 Set<String> valid，用于 O(1) 检查。
     * 如果 endGene 不在 valid 中，说明无法达到目标，直接返回 –1。
     * BFS 初始化
     *
     * 使用队列 Queue<String> 存放当前层（当前步数）所有可达的基因串。
     * 另用一个 Set<String> visited 记录已经访问过的基因串，避免重复遍历。
     * 将起点 startGene 入队、标记已访问；将步数 steps 置 0。
     * 层次遍历
     *
     * 每次遍历一层时，取出当前队列大小 sz，表示这一层共有 sz 个节点要处理；
     * 对每个节点（基因串）进行：
     * 若它等于 endGene，则返回当前 steps。
     * 否则，尝试对这条基因串的 8 个位置，逐个替换为 'A' / 'C' / 'G' / 'T' 中另一种字符，生成所有 “相差 1 位” 的新基因串 next；
     * 如果 next 在 valid 且未被访问过，就将其入队并标记访问。
     * 步数递增
     *
     * 当前层的所有节点都处理完后，steps++，进入下一层。
     * 直到队列为空仍未找到 endGene，返回 –1。
     * @param startGene
     * @param endGene
     * @param bank
     * @return
     */
    public int minMutation(String startGene, String endGene, String[] bank) {
        // 1. 将 bank 放入 Set，便于快速查验
        Set<String> valid = new HashSet<>(Arrays.asList(bank));
        // 如果目标不在 bank 中，无法达成
        if (!valid.contains(endGene)) {
            return -1;
        }

        // 2. BFS 初始化
        char[] genes = new char[] {'A', 'C', 'G', 'T'};
        Queue<String> queue = new ArrayDeque<>();
        Set<String> visited = new HashSet<>();

        queue.offer(startGene);
        visited.add(startGene);
        int steps = 0;

        // 3. 开始层次遍历
        while (!queue.isEmpty()) {
            int sz = queue.size();
            // 处理当前层中的所有节点
            for (int i = 0; i < sz; i++) {
                String curr = queue.poll();
                // 找到目标，返回步数
                if (curr.equals(endGene)) {
                    return steps;
                }
                // 生成所有一位变异的基因串
                char[] currArr = curr.toCharArray();
                for (int pos = 0; pos < currArr.length; pos++) {
                    char old = currArr[pos];
                    for (char g : genes) {
                        if (g == old) continue;  // 跳过相同字符
                        currArr[pos] = g;
                        String next = new String(currArr);
                        // 只有在 bank 且未访问过，才入队
                        if (valid.contains(next) && !visited.contains(next)) {
                            queue.offer(next);
                            visited.add(next);
                        }
                    }
                    currArr[pos] = old;  // 恢复现场，不然每个pos都变了
                }
            }
            // 当前层遍历完，步数+1
            steps++;
        }

        // 队列空了也没找到，返回 -1
        return -1;
    }
}
