package graph;

import java.util.LinkedList;
import java.util.Queue;

/**
 * 题目： 994. 腐烂的橘子  (orangesRotting)
 * 描述：在给定的 m x n 网格 grid 中，每个单元格可以有以下三个值之一：
 * 值 0 代表空单元格；
 * 值 1 代表新鲜橘子；
 * 值 2 代表腐烂的橘子。
 * 每分钟，腐烂的橘子 周围 4 个方向上相邻 的新鲜橘子都会腐烂。
 * 返回 直到单元格中没有新鲜橘子为止所必须经过的最小分钟数。如果不可能，返回 -1 。

 * 链接：https://leetcode.cn/problems/rotting-oranges/

 */
//思想会，代码写不利索
public class OrangesRotting {
    public int orangesRotting(int[][] grid) {
        int rows = grid.length;
        int cols = grid[0].length;
        Queue<int[]> queue = new LinkedList<>();
        int fresh = 0;  // 记录新鲜橘子的数量

        // 将所有腐烂橘子的位置入队，同时统计新鲜橘子的个数
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < cols; j++) {
                if (grid[i][j] == 2) {
                    queue.offer(new int[]{i, j});
                } else if (grid[i][j] == 1) {
                    fresh++;
                }
            }
        }

        // 如果一开始没有新鲜橘子，则不需要任何时间
        if (fresh == 0) {
            return 0;
        }

        int minutes = 0;
        // 四个方向：上、下、左、右
        int[][] directions = {{-1, 0}, {1, 0}, {0, -1}, {0, 1}};

        // BFS 开始，每一层代表一分钟
        while (!queue.isEmpty()) {
            int size = queue.size();
            boolean rotted = false; // 当前这一分钟是否有新鲜橘子被腐烂
            for (int i = 0; i < size; i++) {
                int[] cur = queue.poll();
                for (int[] dir : directions) {
                    int x = cur[0] + dir[0];
                    int y = cur[1] + dir[1];
                    // 检查边界条件和是否为新鲜橘子
                    if (x >= 0 && x < rows && y >= 0 && y < cols && grid[x][y] == 1) {
                        // 腐烂新鲜橘子，并入队后续继续扩散
                        grid[x][y] = 2;
                        fresh--;
                        queue.offer(new int[]{x, y});
                        rotted = true;
                    }
                }
            }
            // 如果这一分钟有橘子腐烂，则分钟数加1
            if (rotted) {
                minutes++;
            }
        }
        // 如果还有新鲜橘子，说明无法全部腐烂，返回 -1
        return fresh == 0 ? minutes : -1;
    }
}
