package graph;
/**
 * 题目： 208. 实现 Trie (前缀树)
 * 描述：Trie（发音类似 "try"）或者说 前缀树 是一种树形数据结构，用于高效地存储和检索字符串数据集中的键。这一数据结构有相当多的应用情景，例如自动补全和拼写检查。
 * 请你实现 Trie 类：
 * Trie() 初始化前缀树对象。
 * void insert(String word) 向前缀树中插入字符串 word 。
 * boolean search(String word) 如果字符串 word 在前缀树中，返回 true（即，在检索之前已经插入）；否则，返回 false 。
 * boolean startsWith(String prefix) 如果之前已经插入的字符串 word 的前缀之一为 prefix ，返回 true ；否则，返回 false 。

 * 链接：https://leetcode.cn/problems/implement-trie-prefix-tree/description/

 */
//没搞懂 学习一下

/**
 * 节点结构
 *
 * 每个 Trie 对象代表一棵子树的根节点，内部维护一个 children[26] 数组，对应 26 个字母。
 * isEnd 标识该节点是否是某个已插入单词的末尾。
 * 插入 (insert)
 * 从根节点出发，依次读取单词每个字符，计算索引 ch - 'a'。
 * 若对应子节点为空则新建，最后将游标节点标记为单词结尾。
 *
 * 查找完整单词 (search)
 * 调用内部方法 searchPrefix，找到单词走到的最后一个节点；
 * 再检查该节点的 isEnd，为 true 时说明完整单词存在。
 *
 * 前缀判断 (startsWith)
 * 仍用 searchPrefix 定位到最后一个节点；
 * 不需要检查 isEnd，只要节点不为 null 就说明有该前缀。
 *
 * 共用方法 (searchPrefix)
 * 将插入和查找里的公共逻辑抽取出来：遍历路径、空指针检查、节点推进。
 */
//https://leetcode.cn/problems/implement-trie-prefix-tree/solutions/98390/trie-tree-de-shi-xian-gua-he-chu-xue-zhe-by-huwt/?envType=study-plan-v2&envId=top-interview-150
class Trie {
    // children 数组保存当前节点的所有子节点，索引 0-25 分别对应 'a'-'z'
    private Trie[] children;
    // isEnd 标志用于标识当前节点是否为一个完整单词的结尾
    private boolean isEnd;

    /**
     * Initialize your data structure here.
     */
    public Trie() {
        // 初始化 children 数组，初始时所有子节点都为 null
        children = new Trie[26];
        // 初始化单词结尾标志为 false
        isEnd = false;
    }

    /**
     * Inserts a word into the trie.
     */
    public void insert(String word) {
        Trie node = this;                // 从根节点开始
        for (int i = 0; i < word.length(); i++) {
            char ch = word.charAt(i);    // 当前字符
            int index = ch - 'a';        // 计算在 children 数组中的索引
            // 如果对应字符的子节点不存在，则新建一个节点
            if (node.children[index] == null) {
                node.children[index] = new Trie();
            }
            // 移动到子节点，继续处理下一个字符
            node = node.children[index];
        }
        // 遍历完所有字符后，将当前节点标记为单词结尾
        node.isEnd = true;
    }

    /**
     * Returns true if the word is in the trie.
     */
    public boolean search(String word) {
        // 尝试查找完整单词对应的节点
        Trie node = searchPrefix(word);
        // 只有节点不为 null 且 isEnd 为 true 才说明单词存在
        return node != null && node.isEnd;
    }

    /**
     * Returns true if there is any word in the trie that starts with the given prefix.
     */
    public boolean startsWith(String prefix) {
        // 只要能找到 prefix 对应的最后一个节点，就说明存在该前缀
        return searchPrefix(prefix) != null;
    }

    /**
     * 查找给定字符串对应的最后一个节点，若路径中断则返回 null。
     */
    private Trie searchPrefix(String prefix) {
        Trie node = this;                // 从根节点开始
        for (int i = 0; i < prefix.length(); i++) {
            char ch = prefix.charAt(i);
            int index = ch - 'a';        // 计算字符索引
            // 若对应子节点不存在，则前缀/单词不在 Trie 中
            if (node.children[index] == null) {
                return null;
            }
            // 移动到子节点，继续检查下一个字符
            node = node.children[index];
        }
        // 成功遍历所有字符，返回最后停留的节点
        return node;
    }
}

