package heap;

import java.util.ArrayList;
import java.util.List;
import java.util.PriorityQueue;

/**
 * 题目： 373. 查找和最小的 K 对数字 (kSmallestPairs)
 * 描述：给定两个以 非递减顺序排列 的整数数组 nums1 和 nums2 , 以及一个整数 k 。
 * 定义一对值 (u,v)，其中第一个元素来自 nums1，第二个元素来自 nums2 。
 * 请找到和最小的 k 个数对 (u1,v1),  (u2,v2)  ...  (uk,vk) 。
 *
 * 示例 1：
 输入: nums1 = [1,7,11], nums2 = [2,4,6], k = 3
 输出: [1,2],[1,4],[1,6]
 解释: 返回序列中的前 3 对数：
 [1,2],[1,4],[1,6],[7,2],[7,4],[11,2],[7,6],[11,4],[11,6]

 * 链接：https://leetcode.cn/problems/find-k-pairs-with-smallest-sums/
 */
//有思路 但是超出时间限制，需要优化
public class KSmallestPairs {
    /**
     * 可以把所有可能的组合看作一个虚拟的二维矩阵：
     *
     * 行代表 nums1[i]
     * 列代表 nums2[j]
     * 每个点 (i,j) 对应的值是 nums1[i] + nums2[j]
     * 这个矩阵的特点是：从左到右、从上到下递增，因为两个数组都升序。
     *
     * 我们不需要全部生成矩阵，而是只维护当前“最小的候选对”，类似 Dijkstra 算法或 BFS：
     *
     * 初始时，把前 k 行的第一列（即 (nums1[i], nums2[0])）放入堆中。因为后面的列比前面的大，我们先选列头。
     * 每次弹出堆顶（最小的和），把它加入结果列表。
     * 如果该对还有下一列（即 (i, j+1) 存在），就把 (i, j+1) 加入堆，作为下一个候选。
     * 这样我们始终只关注当前最小的组合，最多进行 k 次堆操作，高效。
     * @param nums1
     * @param nums2
     * @param k
     * @return
     */
    public List<List<Integer>> kSmallestPairs(int[] nums1, int[] nums2, int k) {
        List<List<Integer>> res = new ArrayList<>();

        // 边界处理：任一数组为空 或 k==0，直接返回空结果
        if (nums1.length == 0 || nums2.length == 0 || k == 0) return res;

        // 小顶堆，存储的是每对数的索引 [i, j]，按对应的和升序排列
        PriorityQueue<int[]> minHeap = new PriorityQueue<>(
                (a, b) -> nums1[a[0]] + nums2[a[1]] - nums1[b[0]] - nums2[b[1]]
        );

        // 初始化：将 nums1 的前 k 个元素与 nums2[0] 组成的数对放入堆中
        // 因为只推进列，所以只需要每一行的第一个元素
        for (int i = 0; i < Math.min(nums1.length, k); i++) {
            minHeap.offer(new int[]{i, 0});  // i 表示 nums1 的索引，j=0 是固定的起点
        }

        // 取出堆顶最小的数对，直到取满 k 个或者堆空为止
        while (k-- > 0 && !minHeap.isEmpty()) {
            int[] pos = minHeap.poll();  // 当前最小的数对在 nums1[pos[0]], nums2[pos[1]]
            int i = pos[0], j = pos[1];

            // 把数对值加入结果集
            res.add(List.of(nums1[i], nums2[j]));

            // 如果当前数对还有下一列，就推进下一列（仍然在第 i 行）
            if (j + 1 < nums2.length) {
                minHeap.offer(new int[]{i, j + 1});
            }
        }

        return res;
    }
}
