package range;

import java.util.Arrays;
import java.util.HashMap;

/**
 * 题目： 452. 用最少数量的箭引爆气球 (findMinArrowShots)
 * 描述：有一些球形气球贴在一堵用 XY 平面表示的墙面上。墙面上的气球记录在整数数组 points ，其中points[i] = [xstart, xend] 表示水平直径在 xstart 和 xend之间的气球。你不知道气球的确切 y 坐标。
 * 一支弓箭可以沿着 x 轴从不同点 完全垂直 地射出。在坐标 x 处射出一支箭，若有一个气球的直径的开始和结束坐标为 xstart，xend， 且满足  xstart ≤ x ≤ xend，则该气球会被 引爆 。可以射出的弓箭的数量 没有限制 。 弓箭一旦被射出之后，可以无限地前进。
 * 给你一个数组 points ，返回引爆所有气球所必须射出的 最小 弓箭数 。

 示例 2：
 输入：points = [[10,16],[2,8],[1,6],[7,12]]
 输出：2
 解释：气球可以用2支箭来爆破:
 -在x = 6处射出箭，击破气球[2,8]和[1,6]。
 -在x = 11处发射箭，击破气球[10,16]和[7,12]。

 * 链接：https://leetcode.cn/problems/minimum-number-of-arrows-to-burst-balloons/
 */
public class FindMinArrowShots {
    /**
     * 一种经典做法是：先按照每个气球的 右端点 从小到大排序；
     * 然后从最左边的气球开始，选它的右端点作为射箭位置，这样能保证这支箭能戳破尽可能多的 “覆盖” 当前右端点的气球；
     * 跳过所有已经被这支箭打到的气球，再对下一个未被覆盖的气球重复相同操作。
     * @param points
     * @return
     */
    public int findMinArrowShots(int[][] points) {
        int n = points.length;
        if (n == 0) {
            // 没有气球，当然不用射箭
            return 0;
        }

        // 1. 按照区间的右端点从小到大排序
        //    这样每次选当前最小的右端点作为射箭位置，
        //    能最大程度地覆盖后续尽可能多的气球
        Arrays.sort(points, (a, b) -> Integer.compare(a[1], b[1]));

        int cnt = 0;    // 箭的数量
        int index = 0;  // 当前未覆盖的气球下标

        // 2. 遍历所有气球，只要还有未戳破的，就再射一支箭
        while (index < n) {
            // 这一支箭射在 points[index][1] 处（当前未处理区间的最小右端）
            int shootPos = points[index][1];
            cnt++;

            // 3. 跳过所有能够被这支箭戳破的气球
            //    条件：它们的左端 ≤ shootPos
            while (index < n && points[index][0] <= shootPos) {
                index++;
            }
        }

        return cnt;
    }
}
