package stack;

import java.util.ArrayDeque;
import java.util.Deque;
import java.util.LinkedList;

/**
 * 题目： 224. 基本计算器 (calculate)
 * 描述：给你一个字符串表达式 s ，请你实现一个基本计算器来计算并返回它的值。
 * 注意:不允许使用任何将字符串作为数学表达式计算的内置函数，比如 eval() 。
 * s 由数字、'+'、'-'、'('、')'、和 ' ' 组成
 * s 表示一个有效的表达式
 * '+' 不能用作一元运算(例如， "+1" 和 "+(2 + 3)" 无效)
 * '-' 可以用作一元运算(即 "-1" 和 "-(2 + 3)" 是有效的)
 *
 * 示例 1：
 输入：s = "(1+(4+5+2)-3)+(6+8)"
 输出：23

 * 链接：https://leetcode.cn/problems/basic-calculator/
 */
//困难题，学思路
public class Calculate {
    //1 + (3 - 2) - (6 - (4 + 5)) = 1 + 3 - 2 - 6 + 4 + 5

    /**
     * 栈是用来管理每一层的符号的。由于括号可以嵌套，所以每一层的计算符号（加号或减号）可能不同。
     * 我们通过栈来保存每一层的符号，当进入一个新的括号时，将当前符号推入栈中，当退出括号时，弹出栈中的符号，回到上一层的符号。
     */
    public int calculate(String s) {
        Deque<Integer> st = new ArrayDeque<>();  // 存放每一层的符号，一个括号表示一层
        int sign = 1;   // 值为1或-1，表示当前数的符号
        st.push(sign);  // 最顶层的符号为+1
        int res = 0;    // 结果
        int number;
        int n = s.length();
        int i = 0;
        char ch;
        while(i < n){
            if(Character.isDigit(ch = s.charAt(i))){
                // 如果为数字，生成数值
                number = 0;
                while(i < n && Character.isDigit(ch = s.charAt(i))){
                    number = number * 10 + (ch - '0');
                    i++;
                }
                // 累加和，真实数字等于符号乘以数值
                res += sign * number;
            }else{
                if(ch == '+'){
                    sign = st.peek();   // 加号，获取当前层的符号
                }else if(ch == '-'){
                    sign = -st.peek();  // 减号，获取当前层的相反符号
                }else if(ch == '('){
                    st.push(sign);      // 左括号，进入新一层，将当前层符号入栈
                }else if(ch == ')'){
                    st.pop();   // 右括号，完成一层的运算，弹出这一层符号
                }
                i++;    // 索引右移
            }
        }
        return res;
    }

    // 测试代码
    public static void main(String[] args) {
        Calculate solution = new Calculate();
        String s = "(1+(4+5+2)-3)-(6+8)";
        System.out.println("输入: " + s);
        System.out.println("输出: " + solution.calculate(s)); // 输出 23
    }
}
