package stack;

import java.util.ArrayDeque;
import java.util.Deque;

/**
 * 题目： 84. 柱状图中最大的矩形
 * 描述：给定 n 个非负整数，用来表示柱状图中各个柱子的高度。每个柱子彼此相邻，且宽度为 1 。
 * 求在该柱状图中，能够勾勒出来的矩形的最大面积。

 * 示例 1：
 输入：heights = [2,1,5,6,2,3]
 输出：10
 解释：最大的矩形为图中红色区域，面积为 10
 *
 * 链接：https://leetcode.cn/problems/largest-rectangle-in-histogram/
 */
//初见不会
public class LargestRectangleArea {
    //暴力法 依次遍历柱形的高度，对于每一个高度分别向两边扩散，求出以当前高度为矩形的最大宽度多少。
    public int largestRectangleArea1(int[] heights) {
        int len = heights.length;
        // 特判
        if (len == 0) {
            return 0;
        }

        int res = 0;
        for (int i = 0; i < len; i++) {

            // 找左边最后 1 个大于等于 heights[i] 的下标
            int left = i;
            int curHeight = heights[i];
            while (left > 0 && heights[left - 1] >= curHeight) {
                left--;
            }

            // 找右边最后 1 个大于等于 heights[i] 的索引
            int right = i;
            while (right < len - 1 && heights[right + 1] >= curHeight) {
                right++;
            }

            int width = right - left + 1;
            res = Math.max(res, width * curHeight);
        }
        return res;
    }
    //动态规划 暴力解优化
    public int largestRectangleArea2(int[] heights) {
        int size = heights.length;
        int[] minLeftIndex = new int[size];
        int[] minRightIndex = new int[size];

        // 记录每个柱子左边第一个小于该柱子的下标
        minLeftIndex[0] = -1; // 初始化，防止死循环
        for (int i = 1; i < size; i++) {
            int t = i - 1;
            // 不断向左寻找，直到找到第一个小于当前柱子的柱子
            while (t >= 0 && heights[t] >= heights[i]) {
                t = minLeftIndex[t]; // 更新为左边第一个小于当前柱子的下标
            }
            minLeftIndex[i] = t;
        }

        // 记录每个柱子右边第一个小于该柱子的下标
        minRightIndex[size - 1] = size; // 初始化，防止死循环
        for (int i = size - 2; i >= 0; i--) {
            int t = i + 1;
            // 不断向右寻找，直到找到第一个小于当前柱子的柱子
            while (t < size && heights[t] >= heights[i]) {
                t = minRightIndex[t]; // 更新为右边第一个小于当前柱子的下标
            }
            minRightIndex[i] = t;
        }

        // 计算最大矩形面积
        int result = 0;
        for (int i = 0; i < size; i++) {
            int width = minRightIndex[i] - minLeftIndex[i] - 1;
            int area = heights[i] * width;
            result = Math.max(result, area);
        }

        return result;
    }
    //单调栈
    public int largestRectangleArea(int[] heights) {
        Deque<Integer>stack=new ArrayDeque<>();
        int max=Integer.MIN_VALUE;
        stack.push(-1);
        for (int i = 0; i < heights.length; i++) {
            while(stack.size()>=2 && heights[i]<heights[stack.peek()]){
                int cur=stack.pop();
                int width=i-stack.peek()-1;
                if(max<width*heights[cur])
                    max=width*heights[cur];
            }
            stack.push(i);
        }
        while(stack.size()>=2){
            int cur=stack.pop();
            int width=heights.length-stack.peek()-1;
            if(max<width*heights[cur])
                max=width*heights[cur];
        }
        return max;
    }
}
