package stack;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.List;

/**
 * 题目： 71. 简化路径 (simplifyPath)
 * 描述：给你一个字符串 path ，表示指向某一文件或目录的 Unix 风格 绝对路径 （以 '/' 开头），请你将其转化为 更加简洁的规范路径。
 *
 * 在 Unix 风格的文件系统中规则如下：
 *
 * 一个点 '.' 表示当前目录本身。
 * 此外，两个点 '..' 表示将目录切换到上一级（指向父目录）。
 * 任意多个连续的斜杠（即，'//' 或 '///'）都被视为单个斜杠 '/'。
 * 任何其他格式的点（例如，'...' 或 '....'）均被视为有效的文件/目录名称。
 * 返回的 简化路径 必须遵循下述格式：
 *
 * 始终以斜杠 '/' 开头。
 * 两个目录名之间必须只有一个斜杠 '/' 。
 * 最后一个目录名（如果存在）不能 以 '/' 结尾。
 * 此外，路径仅包含从根目录到目标文件或目录的路径上的目录（即，不含 '.' 或 '..'）。
 * 返回简化后得到的 规范路径 。

 示例 1：
 输入：path = "/home/"
 输出："/home"
 解释：
 应删除尾随斜杠。

 * 链接：https://leetcode.cn/problems/simplify-path/
 */
public class SimplifyPath {
    public String simplifyPath(String path) {
        Deque<String> stack = new ArrayDeque<>();
        // 按 '/' 分割，连续斜杠会产生空字符串
        for (String dir : path.split("/")) {
            if (dir.isEmpty() || dir.equals(".")) {
                // 当前目录或多余的斜杠，跳过
                continue;
            }
            if (dir.equals("..")) {
                // 返回上一级目录
                if (!stack.isEmpty()) {
                    stack.pop();
                }
            } else {
                // 有效目录名，入栈
                stack.push(dir);
            }
        }

        // 构建结果路径
        if (stack.isEmpty()) {
            return "/";
        }
        StringBuilder result = new StringBuilder();
        while (!stack.isEmpty()) {
            result.insert(0, stack.pop());
            result.insert(0, "/");
        }
        return result.toString();
    }
    public String simplifyPath2(String path) {
        Deque<String> stack = new ArrayDeque<>();
        StringBuilder segment = new StringBuilder();
        // 在末尾加一个 '/'，方便最后一段也能被处理
        String p = path.endsWith("/") ? path : path + "/";

        for (int i = 0; i < p.length(); i++) {
            char c = p.charAt(i);
            if (c == '/') {
                if (segment.length() > 0) {
                    String dir = segment.toString();
                    if ("..".equals(dir)) {
                        if (!stack.isEmpty()) {
                            stack.pop();
                        }
                    } else if (!".".equals(dir)) {
                        stack.push(dir);
                    }
                    segment.setLength(0);
                }
            } else {
                segment.append(c);
            }
        }

        // 构建规范路径
        if (stack.isEmpty()) {
            return "/";
        }
        StringBuilder res = new StringBuilder();
        while (!stack.isEmpty()) {
            res.insert(0, "/" + stack.pop());
        }
        return res.toString();
    }
}
