package substring;

import java.util.*;

/**
 * 题目：239. 滑动窗口最大值 (maxSlidingWindow)
 * 描述：给你一个整数数组 nums，有一个大小为 k 的滑动窗口从数组的最左侧移动到数组的最右侧。你只可以看到在滑动窗口内的 k 个数字。滑动窗口每次只向右移动一位。
 * 返回 滑动窗口中的最大值 。
 *
 * 链接：https://leetcode.cn/problems/sliding-window-maximum/
 *
 示例 1：

 输入：nums = [1,3,-1,-3,5,3,6,7], k = 3
 输出：[3,3,5,5,6,7]
 解释：
 滑动窗口的位置                最大值
 ---------------               -----
 [1  3  -1] -3  5  3  6  7       3
 1 [3  -1  -3] 5  3  6  7       3
 1  3 [-1  -3  5] 3  6  7       5
 1  3  -1 [-3  5  3] 6  7       5
 1  3  -1  -3 [5  3  6] 7       6
 1  3  -1  -3  5 [3  6  7]      7

 示例 2：
 输入：nums = [1], k = 1
 输出：[1]
 */
//普通方法超时 需重做
public class MaxSlidingWindow {
    //优先队列
    public int[] maxSlidingWindow1(int[] nums, int k) {
        int n = nums.length;
        PriorityQueue<int[]> pq = new PriorityQueue<int[]>(new Comparator<int[]>() {
            public int compare(int[] pair1, int[] pair2) {
                return pair1[0] != pair2[0] ? pair2[0] - pair1[0] : pair2[1] - pair1[1];
            }
        });
        for (int i = 0; i < k; ++i) {
            pq.offer(new int[]{nums[i], i});
        }
        int[] ans = new int[n - k + 1];
        ans[0] = pq.peek()[0];
        for (int i = k; i < n; ++i) {
            pq.offer(new int[]{nums[i], i});
            while (pq.peek()[1] <= i - k) {  //如果堆顶元素不在滑动窗口内部，就移除 ，非堆顶元素不管
                pq.poll();
            }
            ans[i - k + 1] = pq.peek()[0];
        }
        return ans;
    }
    //单调队列

    /**
     * 维护一个单调队列。
     在这个滑动窗口最大值的算法中，双端队列的两端各自承担不同的职责：
     队头（Deque Front）：
     队头始终保存当前窗口内最大元素的索引。由于我们维护队列中元素对应的数值是单调递减的，所以队头的元素必定是最大的。当窗口滑动时，如果队头的索引超出窗口范围，会将其移除，以保证队头始终代表当前窗口的最大值。

     队尾（Deque Rear）：
     队尾用于存放候选的元素索引，它们按照数值从大到小的顺序排列。在加入新元素前，会从队尾移除所有比新元素小的元素索引，因为这些较小的元素在未来的窗口中不可能成为最大值。这样既保证了队列的单调递减性质，也为未来的窗口维护了正确的候选顺序。
     */
    public int[] maxSlidingWindow(int[] nums, int k) {
        int n = nums.length;
        Deque<Integer> deque = new LinkedList<>();
        for (int i = 0; i < k; ++i) {
            while (!deque.isEmpty() && nums[i] >= nums[deque.peekLast()]) {
                deque.pollLast();
            }
            deque.offerLast(i);
        }

        int[] ans = new int[n - k + 1];
        ans[0] = nums[deque.peekFirst()];
        for (int i = k; i < n; ++i) {
            while (!deque.isEmpty() && nums[i] >= nums[deque.peekLast()]) {  //队尾元素本来就比队头小，如果新元素比队尾还大，那就没有保留的价值了
                deque.pollLast();
            }
            deque.offerLast(i);
            while (deque.peekFirst() <= i - k) {
                deque.pollFirst();
            }
            ans[i - k + 1] = nums[deque.peekFirst()];
        }
        return ans;
    }
}
