package substring;

import java.util.HashMap;

/**
 * 题目：560. 和为 K 的子数组 (subarraySum)
 * 描述：给你一个整数数组 nums 和一个整数 k ，请你统计并返回 该数组中和为 k 的子数组的个数 。
 *      子数组是数组中元素的连续非空序列。
 * 链接：https://leetcode.cn/problems/subarray-sum-equals-k/
 *
 示例 1：
 输入：nums = [1,1,1], k = 2
 输出：2

 示例 2：
 输入：nums = [1,2,3], k = 3
 输出：2
 */
//只会暴力解
public class SubarraySum {
    //枚举
    public int subarraySum(int[] nums, int k) {
        int count = 0;
        int len = nums.length;
        for (int left = 0; left < len; left++) {
            int sum = 0;
            // 区间里可能会有一些互相抵销的元素
            for (int right = left; right < len; right++) {
                sum += nums[right];
                if (sum == k) {
                    count++;
                }
            }
        }
        return count;
    }
    /**前缀和
     * pre[i]代表 [0,i]内所有元素的和，pre[j]代表 [0,j]内所有元素之和。那么子串[i,j]之和为a[j]-a[i-1]
     */
    public int subarraySum1(int[] nums, int k) {
        int count = 0;        // 用于累计满足条件的子数组个数
        int prefixSum = 0;    // 当前前缀和
        // 哈希表，用于存储前缀和出现的次数，初始化时前缀和为 0 出现 1 次
        HashMap<Integer, Integer> map = new HashMap<>();
        map.put(0, 1);

        // 遍历数组
        for (int num : nums) {
            prefixSum += num;
            // 如果存在前缀和为 prefixSum - k 的情况，
            // 则从那个位置到当前下标的子数组和为 k
            if (map.containsKey(prefixSum - k)) {
                count += map.get(prefixSum - k);
            }
            // 更新哈希表，将当前的前缀和计数加 1
            map.put(prefixSum, map.getOrDefault(prefixSum, 0) + 1);
        }

        return count;
    }
}
