package tree;

import java.util.HashMap;
import java.util.Map;

/**
 * 题目： 437. 路径总和 III  (pathSum)
 * 描述：给定一个二叉树的根节点 root ，和一个整数 targetSum ，求该二叉树里节点值之和等于 targetSum 的 路径 的数目。
 * 路径 不需要从根节点开始，也不需要在叶子节点结束，但是路径方向必须是向下的（只能从父节点到子节点）。

 * 链接：https://leetcode.cn/problems/path-sum-iii/

 */
//前缀和想到了，代码不会写
public class PathSum {
    // 主函数：用于计算路径和等于 targetSum 的路径个数
    public int pathSum(TreeNode root, int targetSum) {
        // 使用一个哈希表来记录前缀和出现的次数
        // key：从根节点到当前节点的累计和（前缀和）
        // value：该前缀和出现的次数
        Map<Long, Integer> prefix = new HashMap<Long, Integer>();
        // 初始化：前缀和为 0 的情况出现 1 次（方便处理从根节点开始的情况）
        prefix.put(0L, 1);
        // 从根节点开始递归查找满足条件的路径
        return dfs(root, prefix, 0, targetSum);
    }

    // dfs函数：递归遍历树，并统计满足路径和为 targetSum 的路径个数
    // 参数说明：
    // root：当前节点
    // prefix：记录前缀和及其出现次数的哈希表
    // curr：当前节点的累计前缀和（从根节点到当前节点的和）
    // targetSum：目标路径和
    public int dfs(TreeNode root, Map<Long, Integer> prefix, long curr, int targetSum) {
        // 终止条件：当前节点为空，返回0条路径
        if (root == null) {
            return 0;
        }

        int ret = 0;
        // 更新当前累计前缀和：加上当前节点的值
        curr += root.val;

        // 检查是否存在某个前缀和，使得：
        // 当前累计和 - 该前缀和 = targetSum
        // 即：从那个节点到当前节点的路径和等于 targetSum
        ret = prefix.getOrDefault(curr - targetSum, 0);

        // 更新哈希表：将当前前缀和的出现次数加1
        prefix.put(curr, prefix.getOrDefault(curr, 0) + 1);

        // 递归遍历左子树和右子树
        ret += dfs(root.left, prefix, curr, targetSum);
        ret += dfs(root.right, prefix, curr, targetSum);

        // 回溯操作：在返回父节点之前，去掉当前节点对前缀和的贡献
        // 防止当前路径的前缀和影响到其他分支的计算
        prefix.put(curr, prefix.getOrDefault(curr, 0) - 1);

        return ret;
    }
}

