package tree;

import java.util.ArrayList;
import java.util.List;

/**
 * 题目： 添加与搜索单词 - 数据结构设计  (WordDictionary)
 * 描述：请你设计一个数据结构，支持 添加新单词 和 查找字符串是否与任何先前添加的字符串匹配 。
 *
 * 实现词典类 WordDictionary ：
 *
 * WordDictionary() 初始化词典对象
 * void addWord(word) 将 word 添加到数据结构中，之后可以对它进行匹配
 * bool search(word) 如果数据结构中存在字符串与 word 匹配，则返回 true ；否则，返回  false 。word 中可能包含一些 '.' ，每个 . 都可以表示任何一个字母。
 *
 示例 1：
 输入：
 ["WordDictionary","addWord","addWord","addWord","search","search","search","search"]
 [[],["bad"],["dad"],["mad"],["pad"],["bad"],[".ad"],["b.."]]
 输出：
 [null,null,null,null,false,true,true,true]

 解释：
 WordDictionary wordDictionary = new WordDictionary();
 wordDictionary.addWord("bad");
 wordDictionary.addWord("dad");
 wordDictionary.addWord("mad");
 wordDictionary.search("pad"); // 返回 False
 wordDictionary.search("bad"); // 返回 True
 wordDictionary.search(".ad"); // 返回 True
 wordDictionary.search("b.."); // 返回 True

 * 链接：https://leetcode.cn/problems/design-add-and-search-words-data-structure/

 */
public class WordDictionary {
    // Trie 树的节点
    private WordDictionary[] children;
    private boolean isEnd;

    public WordDictionary() {
        children = new WordDictionary[26];  // 存储 26 个字母的子节点
        isEnd = false;  // 默认节点不是单词的结尾
    }

    // 添加单词
    public void addWord(String word) {
        WordDictionary node = this;
        for (int i = 0; i < word.length(); i++) {
            int index = word.charAt(i) - 'a';  // 计算字符的索引
            if (node.children[index] == null) {
                node.children[index] = new WordDictionary();  // 如果子节点不存在，就创建一个新节点
            }
            node = node.children[index];  // 移动到当前字符的子节点
        }
        node.isEnd = true;  // 单词的结尾标志
    }

    // 搜索单词，支持通配符 "."
    public boolean search(String word) {
        return searchInNode(word, this);  // 从根节点开始搜索
    }

    // 辅助方法，递归遍历 Trie
    private boolean searchInNode(String word, WordDictionary node) {
        if (word.length() == 0) {
            return node.isEnd;  // 如果单词已遍历完，检查当前节点是否是一个完整单词
        }

        char firstChar = word.charAt(0);
        if (firstChar == '.') {
            // 如果是通配符 '.', 检查每个子节点
            for (int i = 0; i < 26; i++) {
                if (node.children[i] != null && searchInNode(word.substring(1), node.children[i])) {  //word.substring(1):从 word 字符串的第二个字符
                    return true;  // 如果任意一个子节点能匹配，返回 true
                }
            }
            return false;  // 如果所有子节点都无法匹配，返回 false
        } else {
            int index = firstChar - 'a';  // 计算字母的索引
            if (node.children[index] == null) {
                return false;  // 如果当前字符对应的子节点为空，返回 false
            }
            return searchInNode(word.substring(1), node.children[index]);  // 向下递归检查
        }
    }
}
