package trick;
/**
 * 题目： 287. 寻找重复数 (findDuplicate)
 * 描述：给定一个包含 n + 1 个整数的数组 nums ，其数字都在 [1, n] 范围内（包括 1 和 n），可知至少存在一个重复的整数。
 * 假设 nums 只有 一个重复的整数 ，返回 这个重复的数 。
 * 你设计的解决方案必须 不修改 数组 nums 且只用常量级 O(1) 的额外空间。
 *
 示例 1：
 输入：nums = [1,3,4,2,2]
 输出：2

 * 链接：https://leetcode.cn/problems/find-the-duplicate-number/
 */
//不会
public class FindDuplicate {
    /**“按位统计”＋位运算
     * 对于答案 x（重复的数），它在二进制的某一位上是 1，那么这一位上，整个数组里置 1 的次数，一定比从 1 到 n 的所有数里置 1 的次数要多（多出的部分就是重复的那一位贡献的）。
     * 所以我们可以：
     * 对 0..31 每一位，遍历 nums 数组，统计该位上多少个 1：countNums。
     * 同时统计 1 到 n 这 n 个数在该位上多少个 1：countRange。
     * 如果 countNums > countRange，说明重复的数字在这一位上是 1，否则是 0。
     * 最后把所有位拼起来，就是答案。
     * @param nums
     * @return
     */
    public int findDuplicate(int[] nums) {
        int n = nums.length - 1; // nums 中的数都在 [1..n] 里
        int duplicate = 0;
        // 对 0..31 位都做一次统计
        for (int bit = 0; bit < 32; bit++) {
            int mask = 1 << bit;
            int countNums = 0, countRange = 0;
            // 统计数组里这一位的 1 的个数
            for (int x : nums) {
                if ((x & mask) != 0) countNums++;
            }
            // 统计 [1..n] 里这一位的 1 的个数
            for (int i = 1; i <= n; i++) {
                if ((i & mask) != 0) countRange++;
            }
            // 如果数组中多出来，说明重复数这一位是 1
            if (countNums > countRange) {
                duplicate |= mask;
            }
        }
        return duplicate;
    }

    /**
     * 数值范围
     * 因为 nums 大小是 n+1，且所有元素都在 [1..n] 里，必有重复。我们把搜索范围定在 low = 1 到 high = n。
     *
     * 计数判定
     * 对中点 mid = (low + high) / 2，遍历一次数组，统计有多少元素 ≤ mid，记为 cnt。
     *
     * 如果 cnt > mid，说明在 [1..mid] 里「装不下」这么多数（因为最多只有 mid 个不同的数），重复值必然落在这一半区间，于是令 high = mid。
     * 否则，说明重复值在右边区间，令 low = mid + 1。
     * 结束条件
     * 当 low == high 时，区间只剩一个数，它就是答案。
     * @param nums
     * @return
     */
    public int findDuplicate1(int[] nums) {
        int n = nums.length - 1;    // 数值范围在 [1..n]
        int low = 1, high = n;
        while (low < high) {
            int mid = low + (high - low) / 2;
            // 统计 nums 中有多少个数 <= mid
            int cnt = 0;
            for (int x : nums) {
                if (x <= mid) cnt++;
            }
            // 如果“数量”超过了 mid，说明重复值在 [low..mid]
            if (cnt > mid) {
                high = mid;
            } else {
                low = mid + 1;
            }
        }
        // low == high
        return low;
    }

}
