package union_find;
/**
 * 题目： 990. 等式方程的可满足性 (equationsPossible)
 * 描述：给定一个由表示变量之间关系的字符串方程组成的数组，每个字符串方程 equations[i] 的长度为 4，并采用两种不同的形式之一："a==b" 或 "a!=b"。在这里，a 和 b 是小写字母（不一定不同），表示单字母变量名。
 * 只有当可以将整数分配给变量名，以便满足所有给定的方程时才返回 true，否则返回 false。
 *
 示例 1：
 输入：["a==b","b!=a"]
 输出：false
 解释：如果我们指定，a = 1 且 b = 1，那么可以满足第一个方程，但无法满足第二个方程。没有办法分配变量同时满足这两个方程。

 * 链接：https://leetcode.cn/problems/satisfiability-of-equality-equations/
 */
//并查集，第一次不会
public class EquationsPossible {
    public boolean equationsPossible(String[] equations) {
        int[] parent = new int[26];
        for (int i = 0; i < 26; i++) {
            parent[i] = i;
        }
        for (String str : equations) {
            if (str.charAt(1) == '=') {
                int index1 = str.charAt(0) - 'a';
                int index2 = str.charAt(3) - 'a';
                union(parent, index1, index2);
            }
        }
        for (String str : equations) {
            if (str.charAt(1) == '!') {
                int index1 = str.charAt(0) - 'a';
                int index2 = str.charAt(3) - 'a';
                if (find(parent, index1) == find(parent, index2)) {
                    return false;
                }
            }
        }
        return true;
    }
    //合并x,y所属的集合，这里没有做按秩合并，默认合到index2所在集合
    public void union(int[] parent, int index1, int index2) {
        // 先分别找到 index1 和 index2 的根节点，再把 root(index1) 的父指针指向 root(index2)
        parent[find(parent, index1)] = find(parent, index2);
    }
    //查找 index 元素所在集合的根节点（同时做路径压缩）
    public int find(int[] parent, int index) {
        // 当 parent[index] == index 时，说明已经是根节点
        while (parent[index] != index) {
            // 路径压缩：将当前节点直接挂到它父节点的父节点上
            // 这样可以让树变得更扁平，后续查找更快
            parent[index] = parent[parent[index]];
            // 跳到上一级，继续判断是否到根
            index = parent[index];
        }
        // 循环结束时，index 即为根节点下标
        return index;
    }

}
